"""\
Home of the abstract Element base class.
"""
import zLOG
from OFS.SimpleItem import Item
from Globals import Persistent, InitializeClass
from Acquisition import Implicit
from AccessControl import ClassSecurityInfo
from Persistence import PersistentMapping
import string, operator

from Products.CMFCore.CMFCorePermissions import AccessContentsInformation
from ArticlePermissions import ChangeArticles

class AbstractElement(Persistent, Item, Implicit):
    """ Abstract base class for Element objects. """

    security = ClassSecurityInfo()

    sort_order = 0
    clear = 'none'

    def __cmp__(self, other):
        """ Compare function - used when sorting elements """
        if other is not None:
            return cmp(self.sort_order, other.sort_order)
        else:
            return 1

    security.declareProtected(ChangeArticles, 'setSortOrder')
    def setSortOrder(self, order):
        """ set the sort order of this article """
        self.sort_order = order

    security.declareProtected(ChangeArticles, 'apply')
    def apply(self, command):
        """\
        command is expected to be a dictionary.  It gets applied
        to self.edit, and exists to make using Python Scripts
        easier.
        """
        apply(self.edit, (), command)
        self.__changed__(1)
        self.portal_catalog.indexObject(self.aq_parent)

    security.declareProtected(ChangeArticles, 'applyExtra')
    def applyExtra(self, command):
        """\
        command is expected to be a dictionary.  It gets applied
        to self.edit, and exists to make using Python Scripts
        easier.
        """
        apply(self.editExtra, (), command)
        self.__changed__(1)
        self.portal_catalog.indexObject(self.aq_parent)

    def editExtra(self, clear):
        """ Update the value """
        self.float = str(float)

    security.declareProtected(ChangeArticles, 'getEditExtraForm')
    def getEditExtraForm(self):
        " Return the skinned name of the edit form "
        return None

    security.declareProtected(ChangeArticles, 'editableAttributes')
    def editableAttributes(self):
        """\
        Return a list of editable attributes, used by articles
        to build commands to send to the 'edit' command of each
        element, which may vary.

        Requires concrete subclasses to implement the attribute
        '_editableAttributes' which is a tuple of attributes
        that can be edited, for example:

          _editableAttributes = ('value', 'direction',)
        """
        return self._editableAttributes

    security.declareProtected(ChangeArticles, 'editableAttributes')
    def editableExtraAttributes(self):
        """\
        Return a list of editable extra attributes, used by articles
        to build commands to send to the 'editExtra' command of each
        element, which may vary.

        Requires concrete subclasses to implement the attribute
        '_editableExtraAttributes' which is a tuple of attributes
        that can be edited, for example:

          _editableAttributes = ('value', 'direction',)
        """
        return self._editableExtraAttributes

    security.declareProtected(AccessContentsInformation, 'Type')
    def Type(self):
        """\
        Return the Type of Element this object is.  This
        method can be overriden in subclasses, or those
        concrete subclasses must define the 'meta_type'
        attribute.
        """
        return self.meta_type


    security.declareProtected(AccessContentsInformation, 'Description')
    def Description(self):
        """\
        Return a brief but helpful description of the Element type,
        preferably based on the classes __doc__ string.
        """
        strip = string.strip
        split = string.split

        return string.join(             # Sew a string together after we:
            filter(operator.truth,      # Filter out empty lines
                   map(strip,           # strip whitespace off each line
                       split(self.__doc__, '\n') # from the classes doc string
                       )
                   )
            )

    def manage_afterAdd(self, item, container):
        """ set the sort order """
        zLOG.LOG('AbstractElement', zLOG.INFO, 'manage_afterAdd()')
        sort_order = 0
        for e in container.objectValues():
            #zLOG.LOG('AbstractElement', zLOG.INFO, 'manage_afterAdd: elem %s' % (e.getId()))
            if e.getId() != item.getId() and e.sort_order >= sort_order:
                sort_order = (e.sort_order + 1)
        self.sort_order = sort_order

    #def __setstate__(self, dict):
        """\
        Initialize unknown attributes
        """
    #    for name in self._editableAttributes:
    #        if not dict.has_key(name):
    #            dict[name] = ''
    #    self.__dict__ = dict
    #    self.__dicts__.__changed__(1)

InitializeClass(AbstractElement)


