"""
"""

#import os, urllib
import sys, traceback
import zLOG
from Globals import InitializeClass
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import utils
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo, getSecurityManager
from DateTime import DateTime
from Acquisition import aq_base, aq_parent, aq_inner
from ComputedAttribute import ComputedAttribute
from utils import HTMLArticleParser
import string

# Import permission names
from Products.CMFCore import CMFCorePermissions
import ArticlePermissions

# Factory type information -- makes Article objects play nicely
# with the Types Tool (portal_types)

factory_type_information = (
    {'id': 'Article',
     'content_icon': 'article.png',
     'meta_type': 'Portal Article',
     'description': ('Articles are collection of elements '
                     'containing text, images, tables, links etc..'),
     'product': 'CMFArticle',
     'factory': 'addArticle',
     'immediate_view': 'article_edit_form',
     'actions': ({'id': 'view',
                  'name': 'View',
                  'action': 'article_view',
                  'permissions': (CMFCorePermissions.View,)},
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'article_edit_form',
                  'permissions': (ArticlePermissions.ChangeArticles,)},
                 {'id': 'metadata',
                  'name': 'Metadata',
                  'action': 'metadata_edit_form',
                  'permissions': (ArticlePermissions.ChangeArticles,)},
                 {'id': 'elements',
                  'name': 'Elements',
                  'action': 'article_element_form',
                  'permissions': (ArticlePermissions.ChangeArticles,)},
                 {'id': 'subarticles',
                  'name': 'Subarticles',
                  'action': 'article_subarticles_form',
                  'permissions': (ArticlePermissions.ChangeArticles,)}
                 ),                     # End Actions
     },
    )

def addArticle(self, id, title='', REQUEST=None):
    """
    Create an empty article.
    """
    article = Article(id)
    article.id = id
    article.title = title
    sort_order = 0
    for a in self.objectValues(article.meta_type):
        if a.sort_order >= sort_order:
            sort_order = (a.sort_order + 1)
    article.sort_order = sort_order
    self._setObject(id, article)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( 'manage_main' )


class Article(PortalFolder, PortalContent, DefaultDublinCoreImpl):
    """
    Articles are collection of elements
    containing text, images, tables, links etc...
    """
    try:
        __implements__ = ( PortalContent.__implements__
                         , DefaultDublinCoreImpl.__implements__
                         )
    except:
        pass
    meta_type='Portal Article'
    _isDiscussable = 1

    manage_options = ( PortalFolder.manage_options[:3] +
                       ( { 'label'  : 'Dublin Core',
                           'action' : 'manage_metadata',
                           },
                         ) +
                       PortalFolder.manage_options[3:]
                       )

    sort_order = 0
    size = -1

    # Use Zope 2.3 declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)

    _elementTypes = []

    def __init__( self
                , id
                , title=''
                , subject=()
                , description=''
                , contributors=()
                , effective_date=None
                , expiration_date=None
                , format='text/html'
                , language='en-US'
                , rights=''
                ):
        DefaultDublinCoreImpl.__init__( self, title, subject, description
                               , contributors, effective_date, expiration_date
                               , format, language, rights )

    #
    # Workflow interaction
    #

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        " Add self to the workflow and catalog. "
        # Recurse in the children (ObjectManager)
        PortalFolder.manage_afterAdd(self, item, container)
        # Add self to workflow and catalog
        PortalContent.manage_afterAdd(self, item, container)
        # remember to move the initial sorting here

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        " Remove self from the workflow and catalog. "
        # We don't call PortalContent.manage_beforeDelete because
        #  it deals with talkbacks which are subobjects...
        # Remove self from catalog
        if aq_base(container) is not aq_base(self):
            self.unindexObject()
        # Then recurse in the children (ObjectManager)
        PortalFolder.manage_beforeDelete(self, item, container)

    def __cmp__(self, other):
        """ Compare function - used when sorting subarticles """
        if other is not None:
            try:
                return cmp(self.sort_order, other.sort_order)
            except:
                return 1
        else:
            return 1

    # Contentish interface methods
    # ----------------------------

    security.declareProtected(CMFCorePermissions.View, 'icon')
    def icon(self):
        """ For the ZMI """
        return self.getIcon()

    def _index_html(self):
        '''
        Invokes the action identified by the id "view" or the first action.
        '''
        tool = utils.getToolByName( self, 'portal_types' )
        ti = tool.getTypeInfo( self )
        if ti is not None:
            path = ti.getActionById('view', None)
            if path is not None:
                view = self.restrictedTraverse(path)
                return view
            actions = ti.getActions()
            if actions:
                path = actions[0]['action']
                view = self.restrictedTraverse(path)
                return view
            raise 'Not Found', ('No default view defined for type "%s"'
                                % ti.getId())
        else:
            raise 'Not Found', ('Cannot find default view for "%s"'
                                % self.getPhysicalPath())

    security.declareProtected(CMFCorePermissions.View, 'index_html')
    index_html = ComputedAttribute(_index_html, 1)

    security.declareProtected(CMFCorePermissions.View,'Contributors')
    def Contributors(self):
        """  list of authors
        """
        articles = self.objectValues(self.meta_type)

        output = []

        for a in articles:
            contributors = a.Contributors()
            for creator in contributors:
                if creator not in output:
                    output.append(creator)  #add to the list
        creator = self.Creator()
        if creator not in output:
            output.append(creator)  #add to the list
        return output

    def _element_metatype_ids(self):
        result = []
        for mt in self._elementTypes:
            result.append(mt.meta_type)
        return tuple(result)

    security.declareProtected(CMFCorePermissions.View, 'listElements')
    def listElements(self):
        """ Return a list of our element objects """
        objs = self.objectValues(self._element_metatype_ids())
        elements = []
        ms = getToolByName(self, 'portal_membership')
        for obj in objs:
            if ms.checkPermission('View', obj):
             elements.append(obj)
        elements.sort()
        return elements

    security.declareProtected(CMFCorePermissions.View, 'listElementTypes')
    def listElementTypes(self):
        """ Returns a list of the different element types to be used in an article """
        out = []
        for ct in self._elementTypes:
            out.append({
                'name': ct.meta_type,
                'action':'addElement'
                })
        return out

    security.declareProtected(CMFCorePermissions.View, 'all_meta_types')
    def all_meta_types(self):
        """ satisfy copy support """
        return self.listElementTypes()

    security.declareProtected(CMFCorePermissions.View, 'listSubarticles')
    def listSubarticles(self):
        """ Return a list of our subarticles """
        objs = self.objectValues(self.meta_type)
        articles = []
        ms = getToolByName(self, 'portal_membership')
        for obj in objs:
            if ms.checkPermission('View', obj):
             articles.append(obj)
        articles.sort()
        return articles

    security.declareProtected(CMFCorePermissions.View, 'nextArticle')
    def nextArticle(self):
        """ Return the next article object, if any. """
        parent = aq_parent(aq_inner(self))
        if parent.meta_type == self.meta_type:
            for article in parent.listSubarticles():
                if article.sort_order > self.sort_order:
                    return article
        else:
            for a in self.listSubarticles():
                return a

    security.declareProtected(CMFCorePermissions.View, 'previousArticle')
    def previousArticle(self):
        """ Return the next article object, if any. """
        parent = aq_parent(aq_inner(self))
        if parent.meta_type == self.meta_type:
            previous = None
            for article in parent.listSubarticles():
                if article.getId() == self.getId():
                    return previous
                previous = article

    security.declareProtected(CMFCorePermissions.View, 'parentArticle')
    def parentArticle(self):
        """ Return the parent article object, if any. """
        parent = aq_parent(aq_inner(self))
        if parent.meta_type == self.meta_type:
            return parent

    security.declareProtected(ArticlePermissions.ChangeArticles, 'setSortOrder')
    def setSortOrder(self, order):
        """ set the sort order of this article """
        self.sort_order = order

    security.declareProtected(ArticlePermissions.ChangeArticles, 'edit')
    def edit(self, title=None, description=None):
        """ Update meta data about the Article. """
        if title is not None: self.title = title
        self.description = description
        self.portal_catalog.indexObject(self)

    ### Element adding/editing/deleting
    security.declareProtected(ArticlePermissions.ChangeArticles, 'addElement')
    def addElement(self, element_type):
        """ Create a new element in this article """
        zLOG.LOG('Article', zLOG.INFO, 'addElement(%s)' % (element_type))
        element = None
        now = DateTime()
        newid = ('element_%s' % DateTime().strftime('%m%d%y%H%M%S'))
        for et in self._elementTypes:
            if element_type == et.meta_type:
                zLOG.LOG('Article', zLOG.INFO, 'addElement: id %s' % (newid))
                element = et(newid)

        if element is None:
            # No element type matched passed in value
            raise NameError, 'Unknown Element Type: %s' % element_type

        self._setObject(newid, element)
        self.portal_catalog.indexObject(self)
        return newid

    security.declareProtected(ArticlePermissions.ChangeArticles, 'deleteElement')
    def deleteElement(self, element_id):
        """ Delete selected element """
        if type(element_id) is type(''):
            self._delObject(element_id)
        elif type(element_id) in (type(()), type([])):
            for eid in element_id:
                self._delObject(eid)
        self.portal_catalog.indexObject(self)

    security.declarePublic(CMFCorePermissions.View, 'getElement')
    def getElement(self, element_id):
        """ Get the element object """
        return hasattr(self, element_id) and self._getOb(element_id) or None

    security.declareProtected(CMFCorePermissions.AddPortalContent, 'addSubarticle')
    def addSubarticle(self, id):
        """ Add a new subarticle """
        tool = utils.getToolByName( self, 'portal_types' )
        articletype = tool.getTypeInfo(self)
        articletype.constructInstance(self, id)

        return self._getOb(id)



    def PUT_factory( self, name, typ, body ):
        """
           Create an appropriate "skin method".
        """

        if typ == 'text/html': # see if theres any images
            zLOG.LOG('Article', zLOG.INFO, 'PUT_factory - creating parser')
            parser = HTMLArticleParser()
            zLOG.LOG('Article', zLOG.INFO, 'PUT_factory - feeding parser')
            try:
                parser.feed(body)
            except:
                zLOG.LOG('Article', zLOG.INFO, 'PUT_factory: exception %s - %s\n%s' % (sys.exc_type, sys.exc_value, traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
            zLOG.LOG('Article', zLOG.INFO, 'PUT_factory - getting tags')
            parser.close()
            tags = parser.tags
            if tags.has_key('img') and type(tags['img']) == type([]): # we've got images
                zLOG.LOG('Article', zLOG.INFO, 'PUT_factory - got images')
                for img in tags['img']:
                    zLOG.LOG('Article', zLOG.INFO, 'PUT_factory: image %s' % str(img))
            del tags
            del parser
        zLOG.LOG('Article', zLOG.INFO, 'PUT_factory(%s, %s)' % (name, typ))
        for ct in self._elementTypes:
            zLOG.LOG('Article', zLOG.INFO, 'PUT_factory: looking in %s' % (ct.meta_type))
            if(hasattr(ct, 'content_types')):
                zLOG.LOG('Article', zLOG.INFO, 'PUT_factory: %s == %s' % (typ, str(ct.content_types)))
                if typ in ct.content_types:
                    zLOG.LOG('Article', zLOG.INFO, 'PUT_factory: adding %s' % (ct.meta_type))
                    try:
                        elem = apply(ct, (name,))
                        zLOG.LOG('Article', zLOG.INFO, 'PUT_factory: element: %s' % elem.getId())
                        return elem
                    except:
                        zLOG.LOG('Article', zLOG.INFO, 'PUT_factory: exception %s - %s\n%s' % (sys.exc_type, sys.exc_value, traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
                    break

        zLOG.LOG('Article', zLOG.INFO, 'PUT_factory: defaulting...')

        return None # Let NullResource figure it out, then

    def MKCOL_handler(self,id,REQUEST=None,RESPONSE=None):
        """
            Handle WebDAV MKCOL.
        """
        addArticle( id=id, title='' )

    #security.declareProtected( CMFCorePermissions.View, 'get_size' )
    #def get_size(self):
    #    """
    #    return the size of the article
    #    """
    #    if self.size != -1:
    #        return self.size
    #    size = 0
    #    zLOG.LOG('Article.get_size()', zLOG.INFO, 'Trying to get size from: %s' % (self.id))
    #    for element in self.listElements():
    #        #zLOG.LOG('Article.get_size()', zLOG.INFO, 'Trying to get size from:%s %s' % (element.meta_type, element.id))
    #        try:
    #            size = size + element.get_size()
    #        except:
    #            pass
    #            #zLOG.LOG('Article.get_size()', zLOG.INFO, '%s doesnt implement get_size' % element.id)
    #    #for article in self.listSubArticles():
    #    #    size = size + article.get_size()
    #
    #    zLOG.LOG('Article.get_size()', zLOG.INFO, 'Size reached %d' % size)
    #    self.size = size
    #    return self.size

    def SearchableText(self):
        """
            SeachableText is used for full text seraches of a portal.
            It should return a concatanation of all useful text.
        """
        stext = []
        for element in self.listElements():
            stext.append(element.SearchableText())
        return "%s %s %s" % (self.title, self.description, string.join(stext,' '))


# Intialize the Article class, setting up security.
InitializeClass(Article)

from Products.CMFCore.register import registerPortalContent
registerPortalContent(Article,
                      constructors=(addArticle,),
                      action='article_edit_form',
                      icon='article.png')
