"""
    Declare image element class.
"""
import string
import zLOG
from AbstractElement import AbstractElement
from AccessControl import ClassSecurityInfo
from Article import Article
import Globals, interfaces

from Products.CMFCore import CMFCorePermissions
import ArticlePermissions

import OFS.Image


class ImageElement(AbstractElement, OFS.Image.Image):
    """
    An Image file.
    """
    __implements__ = (interfaces.Element,)
    content_types = ('image/png','image/gif','image/jpeg')
    #title = ''
    #file = ''
    #float = ''

    meta_type = 'Image Element'
    security = ClassSecurityInfo()

    _editableAttributes = ('file',)
    _editableExtraAttributes = ('float', 'title', 'clear', 'scale')

    def __init__(self, id):
        self.__name__=id
        zLOG.LOG(self.meta_type, zLOG.INFO, '__init__(%s)' % id)
        self._setId(id)
        zLOG.LOG(self.meta_type, zLOG.INFO, '__init__(%s)' % id)
        OFS.Image.File.__init__(self, id, title='', file = '')

    security.declareProtected(CMFCorePermissions.View, 'hasThumbnails')
    def hasThumbnails(self):
        """ check if the image tag hotfix is installed """
        try:
            from Products.ImageTag_Hotfix import thumb
            return 1
        except:
            return 0

    security.declareProtected(ArticlePermissions.ChangeArticles, 'getEditForm')
    def getEditForm(self):
        """ Return the skinned name of the edit form """
        return 'aimage_edit'

    security.declareProtected(ArticlePermissions.ChangeArticles, 'getEditExtraForm')
    def getEditExtraForm(self):
        """ Return the skinned name of the edit form """
        return 'aimage_edit_extra'

    security.declareProtected(CMFCorePermissions.View, 'getView')
    def getView(self):
        """ Return the skinned name of the edit form """
        return 'aimage_view'

    security.declareProtected(CMFCorePermissions.View, 'hasImage')
    def hasImage(self):
        """ Return true if the image has an image """
        return (self.get_size())

    # I bet some would scream seeing that I override this but I think theres
    # a discrepancy between getId and _setId
    #def getId(self):
    #    """Return the id of the object as a string. This method
    #       should be used in preference to accessing an id attribute
    #       of an object directly. The getId method is public."""
    #    name=getattr(self, 'id', None)
    #    if callable(name):
    #        return name()
    #    if hasattr(self, '__name__'):
    #        return self.__name__
    #    if name is not None:
    #        return name
    #    raise AttributeError, 'This object has no id'

    def _isNotEmpty(self, file):
        """ Do various checks on 'file' to try to determine non emptiness. """
        if not file:
            return 0                    # Catches None, Missing.Value, ''
        elif file and (type(file) is type('')):
            return 1
        elif getattr(file, 'filename', None):
            return 1
        elif not hasattr(file, 'read'):
            return 0
        else:
            file.seek(0,2)              # 0 bytes back from end of file
            t = file.tell()             # Report the location
            file.seek(0)                # and return pointer back to 0
            if t: return 1
            else: return 0

    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests"""
        zLOG.LOG(self.meta_type, zLOG.INFO, 'PUT')
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        type=REQUEST.get_header('content-type', None)

        file=REQUEST['BODYFILE']

        data, size = self._read_data(file)
        content_type=self._get_content_type(file, data, self.__name__,
                                            type or self.content_type)
        self.update_data(data, content_type, size)

        RESPONSE.setStatus(204)
        return RESPONSE

    security.declareProtected(ArticlePermissions.ChangeArticles, 'edit')
    def edit(self, file):
        """ Update the value """
        if file is not None:
            if self._isNotEmpty(file):
                self.manage_upload(file)

            self.setFormat(self.content_type)
            file.close()

    security.declareProtected(ArticlePermissions.ChangeArticles, 'editExtra')
    def editExtra(self, float, title, scale, clear):
        """ Update the value """
        #raise "Getting: %s - %s - %s \nsetting: %s - %s - %d" % (str(float), str(scale), str(clear), self.float, self.clear, self.scale)
        self.title = str(title)
        #self.manage_edit(str(title), self.content_type)
        self.float = str(float)
        self.clear = str(clear)
        self.scale = string.atof(scale)

#    security.declareProtected(CMFCorePermissions.View, 'getElementItems')
#    def getElementItems( self ):
#        """ Return a sequence of element items, used build the article """
#        return self.title is not '' and ((self.title, self.file),) or ()

    def SearchableText(self):
        """
            SearchableText is used for full text searches of a portal.
            It should return a concatanation of all useful text.
        """
        return "%s %s" % (self.title, self.Description())

    def manage_afterAdd(self, item, container):
        """Image has an afterAdd method"""
        zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd(%s, %s)' % (item.getId(), container.getId()))
        OFS.Image.Image.manage_afterAdd(self, item, container)
        zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd(%s, %s)' % (item.getId(), container.getId()))
        try:
            AbstractElement.manage_afterAdd(self, item, container)
            zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd - ok')
        except:
            zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd: exception %s - %s\n%s' % (sys.exc_type, sys.exc_value, traceback.format_list(traceback.extract_tb(sys.exc_traceback))))

    def manage_beforeDelete(self, item, container):
        """Image has a beforeDelete method"""
        OFS.Image.Image.manage_beforeDelete(self, item, container)

Globals.InitializeClass(ImageElement)

# Register as an element type with the Article class
Article._elementTypes.append(ImageElement)
